/* Pxys - An ircu proxyscan, cachehash.c
 *
 * Bob Jenkins's modified hashtab.c to use with m_cache.c
 *
 * $Id: cachehash.h,v 1.4 2002/07/29 00:45:48 mbuna Exp $
--------------------------------------------------------------------
  cache_hcreate  - create a hash table
  cache_hdestroy - destroy a hash table
  cache_hcount   - The number of items in the hash table
  cache_hkey     - key at the current position
  cache_hkeyl    - key length at the current position
  cache_hstuff   - stuff at the current position
  cache_hfind    - find an item in the table
  cache_hadd     - insert an item into the table
  cache_hdel     - delete an item from the table
  cache_hstat    - print statistics about the table
  cache_hfirst   - position at the first item in the table
  cache_hnext    - move the position to the next item in the table
--------------------------------------------------------------------
*/

#ifndef PXYS_CACHEHASH_H
#define PXYS_CACHEHASH_H


#ifndef STANDARD
#include "standard.h"
#endif

#include <sys/types.h>


#define CACHEHASH_KEYLENGTH sizeof(ub4)



/* PRIVATE TYPES AND DEFINITIONS */

struct cachehitem
{
  ub1          *key;      /* key that is hashed */
  /*  ub4           keyl;     length of key -- CCC fixed size */
  time_t        stuff;    /* stuff stored in this cachehitem -- time_t replaces void* */
  ub4           hval;     /* hash value */
  struct cachehitem *next;     /* next cachehitem in list */
};
typedef  struct cachehitem  cachehitem;


struct cachehtab
{
  struct cachehitem **table;   /* hash table, array of size 2^logsize */
  word           logsize; /* log of size of table */
  size_t         mask;    /* (hashval & mask) is position in table */
  ub4            count;   /* how many items in this hash table so far? */
  ub4            apos;    /* position in the array */
  struct cachehitem  *ipos;    /* current item in the array */
  struct reroot *space;   /* space for the cachehitems */
  ub4            bcount;  /* # cachehitems useable in current block */
};
typedef  struct cachehtab  cachehtab;





/* PUBLIC FUNCTIONS */


cachehtab *cache_hcreate(/*_ word logsize _*/);


void  cache_hdestroy(/*_ cachehtab *t _*/);


#define cache_hcount(t) ((t)->count)
#define cache_hkey(t)   ((t)->ipos->key)
#define cache_hstuff(t) ((t)->ipos->stuff)



word  cache_hfind(/*_ cachehtab *t, ub1 *key, ub4 keyl _*/);

word  cache_hfindx(/*_ cachehtab *t, ub1 *key, ub4 keyl, ub4 *ohv _*/);

word  cache_hadd(/*_ cachehtab *t, ub1 *key, ub4 keyl, void *stuff _*/);

word  cache_haddx(/*_ cachehtab *t, ub1 *key, ub4 keyl, void *stuff, ub4 hval _*/);

word  cache_hdel(/* cachehtab *t */);

word  cache_hfirst(/*_ cachehtab *t _*/);

#define cache_hnext(t) \
  ((!(t)->ipos) ? FALSE :  \
   ((t)->ipos=(t)->ipos->next) ? TRUE : cache_hnbucket(t))

word cache_hnbucket(/*_ cachehtab *t _*/);

void cache_hstat(/*_ cachehtab  *t, ub4* obuckets, ub4 *oitems _*/);

#endif   /* PXYS_CACHEHASH_H */
