#ifndef PXYS_SEM_IS_DEF
#define PXYS_SEM_IS_DEF
/* $Id: pxys_sem.h,v 1.3 2002/11/15 20:18:38 mbuna Exp $
 *
 * University of Bordeaux I, France
 * http://dept-info.labri.u-bordeaux.fr/
 * Semaphore implementation with pthreads
 *
 * Stephane Thiell <mbuna@undernet.org>
 * 20021115 Added native mach semaphore support (eg. for Darwin BSD)
 *          Added timedwait support
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif
#ifdef HAVE_MACH_TASK_H
#include <mach/task.h>
#endif
#ifdef HAVE_MACH_SEMAPHORE_H
#include <mach/semaphore.h>
#endif

#if defined(HAVE_SEMAPHORE_CREATE) && defined(HAVE_SEMAPHORE_SIGNAL) && defined(HAVE_SEMAPHORE_WAIT)
#define PXYS_SEM_MACH
#endif


#ifdef PXYS_SEM_MACH

/* Mach impl. */
struct pxys_sem_struct {
  semaphore_t sem;
};

typedef mach_timespec_t pxys_sem_timespec_t;

#else

/* pthread impl. for primitive kernels ;-) */
struct pxys_sem_struct {
  pthread_mutex_t mutex;
  pthread_cond_t cond;
  int value;
};

typedef struct timespec pxys_sem_timespec_t;

#endif

typedef struct pxys_sem_struct pxys_sem_t;

extern void pxys_sem_init(pxys_sem_t *sem, int value);

extern void pxys_sem_destroy(pxys_sem_t *sem);

extern void pxys_sem_P(pxys_sem_t *sem);

extern int  pxys_sem_timedP(pxys_sem_t *sem, pxys_sem_timespec_t *wait_time);

extern void pxys_sem_V(pxys_sem_t *sem);

#endif /* PXYS_SEM_IS_DEF */
