/* Pxys - An IRC ProxyScan, m_clientq.c
 * 
 * Copyright (C) 2001,2002 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * Purpose:
 *  It handles the implementation of the "nicks" queue of recent
 *  connected users on the network. It is used by the m_array->fetchitem
 *  method.
 * Implementation:
 *  Static circle queue implemented on a continuous buffer.
 * Compexity:
 *  clientq_drop  | O(n)
 *  clientq_queue | O(1) if not calling clientq_drop (rare)
 *  clientq_deque | O(1)
 *  clientq_stats | O(1)
 *
 * $Id: m_clientq.c,v 1.10 2002/04/21 16:05:54 mbuna Exp $
 */

#include "m_clientq.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "conf.h"
#include "m_structs.h"
#include "ms_msgout.h"
/* xmalloc at the bottom of include files */
#include "xmalloc.h"

extern scanstats_s *scanstats;

#if CLIENTQ_SIZE <= 0
#error bad value for CLIENTQ_SIZE
#endif

static int aepos, aesize;

/* pointers */
static listelement *aelem;
static listelement *listptr;
static listelement *listq;

/* Pre-allocate */
void
clientq_init()
{
   listptr = listq = NULL;
   aepos = aesize = 0;
   aelem = (listelement *)xcalloc(CLIENTQ_SIZE, sizeof(listelement));
}

void
clientq_destroy()
{
   xfree(aelem);
}


void
clientq_drop(int n)
{
  while (n--)
    clientq_deque(NULL);
}


/* cliyxx != NULL */
void
clientq_queue(const yxx * cliyxx)
{
   listelement * lp = listptr;
   
   if (aesize >= CLIENTQ_SIZE) {
      sendto_console("[%s] ClientQ UNDERRUN - removing %d users from queue\r\n", Me->id, CLIENTQ_SIZE/10);
      clientq_drop(CLIENTQ_SIZE/10);
   }
   
   if (lp != NULL) {
      lp = listq;
      lp->link = &aelem[aepos];
      lp = lp->link;
   } else {
      listptr = &aelem[aepos];
      lp = listptr;
   }
   
   lp->link = NULL;
   memcpy(&lp->cliyxx, cliyxx, sizeof(yxx));
   
   listq = lp;
   
   aepos = (aepos < CLIENTQ_SIZE - 1) ? aepos + 1 : 0 ;
   aesize++;
}


int
clientq_deque(yxx ** out_cliyxx)
{
   if (listptr) {
      if (out_cliyxx) /* Allow NULL, no output */
         *out_cliyxx = &listptr->cliyxx;
      listptr = listptr->link;
      aesize--;
      return 1;
   }
   return 0; /* nothing in queue */
}


void
clientq_stats(int* osize, int* oaesize)
{
   *osize = CLIENTQ_SIZE;
   *oaesize = aesize;   
}


/* m_clientq.c */


