/* Pxys - An IRC ProxyScan, mp_proxy.c
 * 
 * Copyright (C) 2001, 2002 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * 
 * Module checking proxy/webserver tunelling vulnerability.
 *
 * $Id: mp_proxy.c,v 1.28 2002/12/16 01:10:17 mbuna Exp $
 */

#define HTTP_CONNECT_REQUEST "CONNECT %s:%d/ HTTP/1.1\n" \
                             "Host:%s:%d\n" \
                             "User-Agent: pxys/" VERSION "\n" \
                             "\n"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifdef HAVE_SYS_POLL_H
#include <sys/poll.h>
#endif

#include "pxys_sem.h"

#include "typedefs.h"
#include "conf.h"
#include "mp_proxy.h"
#include "mp_listener.h"
#include "mp_utils.h"
#include "ms_globals.h"
#include "ms_msgout.h"
#include "m_log.h"
#include "m_scanner.h"
#include "debug.h"
#include "conf.h"
#include "standard.h" /* TRUE/FALSE */
#include "mp_poll.h"   /* poll macros */

#include <errno.h>

extern int SPort;

extern pthread_mutex_t mutex_scanext;

extern int unreach_count;

#ifdef DETECT_APACHE_AGENT
static int parse_agent_io(char *data, int size);
#endif

void
scan_proxy(scanthread_s * sth)
{
   int err, nfds, i;
   struct pollfd *pfd;
   char request[96];
   pxys_sem_timespec_t timewait;
   
   time_t now;
   unsigned int length;
   Boolean oneinprogress = false;
   Boolean stilloneinprogress = false;
   Boolean timed_out;
   
   /* - Build http proxy CONNECT request */
   if (Me->listeneraddr == htonl(INADDR_ANY))
      snprintf(request, sizeof(request), HTTP_CONNECT_REQUEST,
              Me->hostip, SPort, Me->hostip, SPort);
   else
      snprintf(request, sizeof(request), HTTP_CONNECT_REQUEST,
              Me->listener, SPort, Me->hostip, SPort);
           
   scanItems_init(sth, POLLWRITEFLAGS, (unsigned short)sth->port);
   
   for (;;) {
      usleep(sth->counter * 2500);
      nfds = poll(sth->poll_fds, sth->counter, POLL_TIMEOUT);
      
      if (nfds <= -1) {
         if (errno == EINTR)
            continue;
         log_error("scan_proxy: poll error %d\n", errno);
         break;
      }
      
      /* Timeout checking */
      time(&now);
      timed_out = true;
      for (i = 0; i < sth->counter; ++i) {
         struct scanItem *sci = &sth->item[i];
         
         if (sci->state & (SIT_ERROR|SIT_FOUND|SIT_INPROGRESS))
            continue;
         
         if (now >= sth->item[i].timeout)
            sci->state |= SIT_TIMEOUT;
         
         timed_out = timed_out && (sci->state & SIT_TIMEOUT);
      }
      
      /* All fds are timed out, exiting. */
      if (timed_out)
         break;
      
      pfd = sth->poll_fds;
      
      for (i = 0; nfds && (i < sth->counter); ++i, ++pfd) {
         struct scanItem *sci = &sth->item[i];
         
         if (pfd->revents & POLLERRORS) {
            log_error("scan_proxy: POLLERRORS %s", inet_ntoa(sci->addr));
            sci->state |= SIT_ERROR;
            DISABLE_PFD(pfd);
            nfds--;
            continue;
         }
         
         if (sci->state & (SIT_ERROR|SIT_FOUND)) /* do not ignore SIT_INPROGRESS *
            continue;                             * we might want to recv something.. */
         
         if ((err = get_sockerr(pfd->fd))) {
            if (err == EHOSTUNREACH) {
               unreach_count++;
         	   sci->state |= SIT_UNREACH;
         	   sci->dont_cache = 1;
         	   log_write(LOGID_UNREACH, "%s", inet_ntoa(sci->addr));
         	} else if (err != ECONNREFUSED) {
               log_error("scan_proxy: get_sockerr %s [%d] (%s)",
                    inet_ntoa(sci->addr), err, strerror(err));
            }
            sci->state |= SIT_ERROR;
            DISABLE_PFD(pfd);
            nfds--;
            continue;
         }
         
         if ((pfd->events & POLLWRITEFLAGS) && (pfd->revents & POLLWRITEFLAGS)) {
            /* Two step socket write */
            if (IsConnecting(sci)) {    /* Are we really connected? */
               SetConnected(sci);
               pfd->events = POLLWRITEFLAGS; /* We want to write */
               pfd->revents = 0;
               continue;
            }
            
            /* Write should not block or fail... let's go. */
            
            err = send_nonb(pfd->fd, request, strlen(request) + 1, &length);
            if (err < 0 || (err == 0 && sci->retry-- <= 0)) {
               Debug((DBLMAX, "scan_proxy: send_nonb failed"));
               sci->state |= SIT_ERROR;
               DISABLE_PFD(pfd);
               nfds--;
               continue;
            } else if (err == 0) {
               pfd->events = POLLWRITEFLAGS; /* We want to try again */
               pfd->revents = 0;
               continue;
            }
            
            /* Set a new timeout delay. */
            time(&sci->timeout);
            sci->timeout += MyPerform->timeout_send;
            
            sci->state |= SIT_INPROGRESS;
            oneinprogress = true; /* to avoid items check (faster if nothing sent) */
#ifdef DETECT_APACHE_AGENT
            if (sth->port == 80) {
               pfd->events = POLLREADFLAGS;
            } else {
               pfd->events = 0;
               DISABLE_PFD(pfd);
               nfds--;
            }
#else
            pfd->events = 0;
            DISABLE_PFD(pfd);
            nfds--;
#endif
         }
#ifdef DETECT_APACHE_AGENT
         else  if ((pfd->events & POLLREADFLAGS) && (pfd->revents & POLLREADFLAGS)) {
            unsigned int count_out;
            int agent_detect;
            char agent_buf[512];
            
            if (sth->port != 80)
               continue;
            
            if (recv_nonb(pfd->fd, agent_buf, sizeof(agent_buf), &count_out) == 1) {
               if (count_out >= sizeof(agent_buf))
                  count_out = sizeof(agent_buf) - 1;
               agent_buf[count_out] = '\0';
               
               agent_detect = parse_agent_io(agent_buf, sizeof(agent_buf));
               
               if (agent_detect)
                  log_write(LOGID_MISC, "%s:%u %s", inet_ntoa(sci->addr), sth->port, agent_buf);
               else
                  log_error("%s:%u parse_agent_io() failed", inet_ntoa(sci->addr), sth->port);	  
            }
            pfd->events = 0;
            DISABLE_PFD(pfd);
            nfds--;
         }
#endif
      }
   }
   
   /* If we were able to send at least one request to the http proxy server,
    * this thread has to wait for the reply (listener will notify us).
    */
   if (oneinprogress) {
      stilloneinprogress = false;
      for (i = 0; i < sth->counter; ++i) {
         if ((sth->item[i].state & SIT_INPROGRESS)
            && !(sth->item[i].state & SIT_RECEIVED)) {
            stilloneinprogress = true;
            break;
         }
      }
      
      if (stilloneinprogress) {
         pxys_sem_init(&sth->pxySem, 0); /* Create semaphore */
         
         time((time_t*)&timewait.tv_sec);
         timewait.tv_sec += MyPerform->timeout_listen + 1;
         timewait.tv_nsec = 0;
         
         sth->state |= SCANTHREAD_TIMEWAIT;
         
         do {
            err = pxys_sem_timedP(&sth->pxySem, &timewait);
            if (err == ETIMEDOUT) {
               Debug((DBLMAX, "scan_proxy: timeout"));
               break;
            } else if (err != 0) {
               log_error("mp_proxy: pxys_sem_timedP err=%d", err);
               break;
            }
            
            Debug((DBLMAX, "scan_proxy: Reply received!!!"));
            
            stilloneinprogress = false;
            for (i = 0; i < sth->counter; ++i) {
               if ((sth->item[i].state & SIT_INPROGRESS)
                  && !(sth->item[i].state & SIT_RECEIVED)) {
                  stilloneinprogress = true;
                  break;
               }
            }
         } while (stilloneinprogress);
      
         sth->state &= ~SCANTHREAD_TIMEWAIT;
         pxys_sem_destroy(&sth->pxySem);
      }
   }
   scanItems_closeall(sth);
}

#ifdef DETECT_APACHE_AGENT
/* data is null terminated */
static int
parse_agent_io(char *data, int size)
{
   char c, *p = data;
   int result = FALSE;
   
   assert (data != NULL);
   
   /* Only keep http header */
   while ((c = *p++)) {
      if (c == '\n' && *p == '\n') {
         *(p - 1) = '\0';
         break;
      }
   }
   
   if (*p && ((p = strstr(data, "Server:"))
      || (p = strstr(data, "User-Agent:"))
      || (p = strstr(data, "Proxy-Agent:")))) {
      memmove(data, p, size - (p - data));
      p = data;
      while ((c = *p++)) {
         if (c == '\n') {
            *(p - 1) = '\0';
            break;
         }
      }
      if (*data)
         result = TRUE;
   }
   return result;
}
#endif

/* m_proxy.c */
